/**
 * Copyright (C) 2022 Xilinx, Inc
 * Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

// Sub Commands
#include "SubCmdExamine.h"
#include "SubCmdReset.h"
#include "SubCmdValidate.h"

// Supporting tools
#include "../common/MAMain.h"
#include "../common/SubCmd.h"

//#include "jsf_mautil.h"

// System include files
#include <boost/filesystem.hpp>
#include <exception>
#include <iostream>
#include <string>
#include <string_view>

using namespace std::literals::string_view_literals;

// Program entry
int main(int argc, char** argv) {
  // -- Build the supported subcommands
  SubCmdsCollection subCommands;
  const std::string executable = "mautil";

  {
    // Syntax: SubCmdClass( IsHidden, IsDepricated, IsPreliminary)
    subCommands.emplace_back(std::make_shared<SubCmdExamine>(false, false, false));
    subCommands.emplace_back(std::make_shared<SubCmdValidate>(false, false, false));
    subCommands.emplace_back(std::make_shared<SubCmdReset>(false, false, false));
    //subCommands.emplace_back(std::make_shared<  SubCmdProgram  >(false, false, false));
    //subCommands.emplace_back(std::make_shared< SubCmdConfigure >(false,  false, false));

    // Parse sub commands from json files
    //populateSubCommandsFromJSON(subCommands, executable);

    //subCommands.emplace_back(std::make_shared< SubCmdAdvanced >(true,  false, true ));
  }

  for (auto& subCommand : subCommands) {
    subCommand->setExecutableName(executable);
  }

  // -- Program Description
  const std::string description =
      "The AMD MA35 (R) Board Utility (mautil) is a standalone command line utility that"
      " is included with the MA35 installation package. It includes"
      " multiple commands to validate and identify the installed MA35 card(s) along with"
      " additional card details including PCIe (R), telemetry and system"
      " information.\n\nThis information can be used for both card administration and"
      " application debugging.";

  // -- Ready to execute the code
  try {
    main_(argc, argv, executable, description, subCommands);
    return 0;
  } catch (const std::exception& e) {
    if ("Operation Cancelled"sv != e.what()) {
      std::cerr << "  ERROR: " << e.what() << std::endl;
    }
  } catch (...) {
    std::cerr << "ERROR Occurred: " << std::endl;
  }

  return 1;
}
