// Copyright (C) 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#include <jsf_helpers.h>

namespace jsf_helpers {

NodePcieInfo::NodePcieInfo(const std::string& Name, const pci_dev& Device) : jsf_int::NodeJson(Name, {}) {
  char buf[7];
  snprintf(buf, sizeof(buf), "0x%04x", Device.vendor_id);
  m_value["vendor"] = buf;
  snprintf(buf, sizeof(buf), "0x%04x", Device.device_id);
  m_value["device"] = buf;
};

std::vector<pci_dev> GetPciDevices(int Vendor, int Device) {
  pci_filter filter;
  pci_filter_init(nullptr, &filter);
  filter.vendor = Vendor;
  filter.device = Device;
  auto pacc     = pci_alloc();
  pci_init(pacc);
  pci_scan_bus(pacc);
  std::vector<pci_dev> res;
  for (auto dev = pacc->devices; dev; dev = dev->next) {
    if (!pci_filter_match(&filter, dev)) {
      continue;
    }
    pci_fill_info(dev, PCI_FILL_IDENT | PCI_FILL_CLASS | PCI_FILL_CAPS | PCI_FILL_EXT_CAPS | PCI_FILL_BASES);
    res.push_back(*dev);
  }
  pci_cleanup(pacc);
  return res;
}

std::string DbdfString(const pci_dev& Device) {
  char buf[16];
  snprintf(buf, sizeof(buf), "%04x:%02x:%02x.%d", Device.domain, Device.bus, Device.dev, Device.func);
  return buf;
}

} // namespace jsf_helpers
